﻿

KA_RT_Buttons = {
	LastPage = { };
	QuickLooter = { "disenchanted", "bank" };
};


function KA_RT_Buttons:ShowFrame( frameName, type, raidid, itemid, what, zone )
	local frame = _G[frameName];
	if not frame then return nil end;
	if type then frame.type = type; end
    if raidid then frame.raidid = raidid; end
    if itemid then frame.itemid = itemid; end
    if what then frame.what = what; end
    if zone then frame.zone = zone; end
    frame:Show();
end

function KA_RT_Buttons:AddItemShowFrame(  )
	if not this or not this.value then return end; 
	local value = this.value;	
	local lootid = nil;
	if not value.hasArrow then HideDropDownMenu(1); end
	if value.getLootID then 
		lootid = KA_RaidTracker_GetLootId( value.raidid, value.itemplayer, value.itemitemid, value.itemtime );
	end		    
    self:ShowFrame( value.frameName, value.type, value.raidid, lootid or value.itemid, value.what, value.zone )
end

function KA_RT_Buttons:AddItem( level, text, key, frameName, type, func, value, info )
	info = info or { };
	value = value or { };
	func = func or function	()
		KA_RT_Buttons:AddItemShowFrame( );
	end;
			
	if ( text and key ) then
		text = text .. "("..key..")";
	end
   	if frameName then value.frameName = frameName; end
    if type then value.type = type; end

	if not info.text then info.text = text end
	if not info.value then info.value = value end
	if not info.func then info.func = func end

	UIDropDownMenu_AddButton(info, level);
	return info;
end

function KA_RT_Buttons:GetPlayerGroups( raidid )
    local raid = KARaidTrackerDB.RaidLog[raidid];
	if not raid then return { }; end 
	-- make list without duplicates: indexes
	local indexes = { };
	local groups = { };
	local found = nil;
	
	for k, v in pairs(raid["Join"]) do
		if v["player"] then
			found = false;
			for k2, v2 in pairs(indexes) do
				if v2 == v["player"] then
					found = true;
					break;
				end
			end
			if not found then
				tinsert(indexes, v["player"]);
			end
		end
	end
	table.sort(indexes);
	-- hash the list by first letter
	for k, v in pairs(indexes) do
		local group = (KarmaUtility.SpecialCharsTable[strsub(v, 1, 2)] or v);						-- if a special char get that
		group = strupper(strsub(group, 1, 1));
		if not groups[group] then
			groups[group] = { };
		end
		tinsert(groups[group], v);
	end	
	return groups;
end


-- Rarity Button
KA_RT_RarityDropDown = {
	labels = {
		"|c009d9d9dPoor|r",					-- Grey = 9d9d9d
		"|c00ffffffCommon|r",				-- White = ffffff
		"|c001eff00Uncommon|r",				-- Green = 1eff00
		"|c000070ddRare|r",					-- Blue = 0070dd
		"|c00a335eeEpic|r",					-- Purple = a335ee
		"|c00ff8000Legendary|r",			-- Orange = ff8000
		"|c00e6cc80Artifact",				-- Red e6cc80
	};
}

function KA_RT_RarityDropDown:OnUpdate( )
	local id = KARaidTrackerDB.SortOptions[ RaidTrackerFrame.type .. "filter" ];
	UIDropDownMenu_SetSelectedID( KA_RT_RarityDropDownFrame, id );
	KA_RT_RarityDropDownFrameText:SetText( self.labels[ id ] );
end

function KA_RT_RarityDropDown:OnLoad( )
	local initialize = function ( )
		local dropDownOnClick = function ( )
			KARaidTrackerDB.SortOptions[ RaidTrackerFrame.type .. "filter" ] = this:GetID();
			KA_RaidTracker_UpdateView( );
		end
		for i,v in pairs( KA_RT_RarityDropDown.labels ) do
			UIDropDownMenu_AddButton( { text = v, func = dropDownOnClick } );	 
		end
	end
	UIDropDownMenu_Initialize( this, initialize );
	UIDropDownMenu_SetSelectedID( KA_RT_RarityDropDownFrame, 1 );
end


-- Raid Item Button
function KA_RT_RaidItemTemplate_RightClickMenu_Toggle( frame )
	local menu = getglobal(frame:GetName().."RightClickMenu");
	menu.point = "TOPLEFT";
	menu.relativePoint = "BOTTOMLEFT";
	ToggleDropDownMenu( 1, nil, menu, "cursor", 0, 0 );
end

local function KA_RT_RaidItemTemplate_RaidSetZone( )
	local value = this.value;
	if not value then return end
	KARaidTrackerDB.RaidLog[value.raidid]["zone"] = value.zone;
	KA_RaidTracker_Update();
	KA_RaidTracker_UpdateView();
end

function KA_RT_RaidItemTemplate_RightClickMenu_Initialize( frame, level )
	if not level then return end
    local db = KARaidTrackerDB

	local checked = nil;
	
	if level == 1 then
		local raidid = frame:GetID() + FauxScrollFrame_GetOffset(KA_RaidTrackerListScrollFrame);
		local curRaid = db.RaidLog[raidid];
		
		KA_RT_Buttons:AddItem( level, "Edit Start", curRaid["key"], "KA_RaidTrackerEditNoteFrame", "time", nil,
			{raidid = raidid, what = "raidstart"} );
		KA_RT_Buttons:AddItem( level, "Edit End", curRaid["End"], "KA_RaidTrackerEditNoteFrame", "time", nil,
			{raidid = raidid, what = "raidend"} );				

		checked = nil;
		if curRaid["zone"] then
			for k, v in pairs(KA_RaidTracker_Zones) do
				if curRaid["zone"] == v then checked = 1; break end
			end
		end
		KA_RT_Buttons:AddItem( level, "Edit Zone", curRaid["zone"], "KA_RaidTrackerEditNoteFrame", "zone", nil, 			
			{opt = "raid_zones", raidid = raidid}, { hasArrow = 1, checked = checked } );				
	
		KA_RT_Buttons:AddItem( level, "Edit Note", curRaid["note"], "KA_RaidTrackerEditNoteFrame", "raidnote", nil,
			{raidid = raidid} );				
		KA_RT_Buttons:AddItem( level, "Show DKP String", nil, nil, nil,
			function() KA_RaidTrackerGenerateDkpLink( this.value.raidid ); end,
			{raidid = raidid} );					
		
	elseif level == 2 then
		local value = frame.value; 
		if not value then return end;
			
		if frame.value.opt == "raid_zones" then
			for k, v in pairs(KA_RaidTracker_Zones) do
				if db.RaidLog[value.raidid]["zone"] == v then checked = 1; else checked = nil end
				KA_RT_Buttons:AddItem( level, v, nil, nil, nil, 
					function()	KA_RT_RaidItemTemplate_RaidSetZone( ) end,
					{ raidid = value.raidid, zone = v}, { checked = checked } );				
			end
			if db.RaidLog[value.raidid]["zone"] == nil then checked = 1; else checked = nil end
			KA_RT_Buttons:AddItem( level, "None", nil, nil, nil,
				function()	KA_RT_RaidItemTemplate_RaidSetZone( ) end,
				{raidid = value.raidid}, { checked = checked } );
									
			UIDropDownMenu_AddButton( { disabled = 1 }, level );
				
			for k, v in pairs(db.CustomZoneTriggers) do
				if db.RaidLog[this.value.raidid]["zone"] == v then checked = 1; else checked = nil end
				KA_RT_Buttons:AddItem( level, v, nil, nil, nil, thisFunc,
					{ raidid = value.raidid, zone = v}, { checked = checked } );			
			end

		end
	end
end


-- Loot Item Button

function KA_RT_LootItemTemplate_RightClickMenu_Toggle( frame )
	local menu = getglobal(frame:GetParent():GetName().."RightClickMenu");
	menu.point = "TOPLEFT";
	menu.relativePoint = "BOTTOMLEFT";
	ToggleDropDownMenu(1, nil, menu, "cursor", 0, 0);
end

local function KA_RT_LootItemTemplate_LootSetBoss( )
	local value = this.value;
	if not value then return end
	local lootid = KA_RaidTracker_GetLootId( value.raidid, value.itemplayer, value.itemitemid, value.itemtime );
	KARaidTrackerDB.RaidLog[value.raidid]["Loot"][lootid]["boss"] = value.boss;
	KA_RaidTracker_Update();
	KA_RaidTracker_UpdateView();
end

local function KA_RT_LootItemTemplate_LootSetLooter( )
	local value = this.value;
	if not value then return end
	local lootid = KA_RaidTracker_GetLootId( value.raidid, value.itemplayer, value.itemitemid, value.itemtime );
	KARaidTrackerDB.RaidLog[value.raidid]["Loot"][lootid]["player"] = value.player;
	KA_RaidTracker_Update();
	KA_RaidTracker_UpdateView();
end

function KA_RT_LootItemTemplate_RightClickMenu_Initialize( frame, level )
	if not level then return end
	
	local util = KarmaUtility
	local db = KARaidTrackerDB
	
	local checked = nil;
	
	if level == 1 then
		local raidid = frame:GetParent().raidid;
		local itemid = frame:GetParent().itemid;
	
		local curRaid = db.RaidLog[raidid];
		local itemitemid = curRaid["Loot"][itemid]["item"]["id"];
		local itemtime = curRaid["Loot"][itemid]["time"];
		local itemplayer = curRaid["Loot"][itemid]["player"];

		KA_RT_Buttons:AddItem( level, "Edit Looter", nil, "KA_RaidTrackerEditNoteFrame", "looter", nil,
			{ opt="quick_looter", raidid=raidid, itemid=itemid, itemitemid=itemitemid, itemtime=itemtime, itemplayer=itemplayer, cplayer=curRaid["Loot"][itemid]["player"] },
			{ hasArrow = 1, getLootID = 1 } );
		KA_RT_Buttons:AddItem( level, "Edit Costs", curRaid["Loot"][itemid]["costs"], "KA_RaidTrackerEditCostFrame", "itemcost", nil,
			{ raidid=raidid, itemid=itemid, itemitemid=itemitemid, itemtime=itemtime, itemplayer=itemplayer },
			{ getLootID = 1 } );
		KA_RT_Buttons:AddItem( level, "Edit Count", curRaid["Loot"][itemid]["item"]["count"], "KA_RaidTrackerEditNoteFrame", "itemcount", nil,
			{ raidid=raidid, itemid=itemid, itemitemid=itemitemid, itemtime=itemtime, itemplayer=itemplayer },
			{ getLootID = 1 } );
		KA_RT_Buttons:AddItem( level, "Edit Time", nil, "KA_RaidTrackerEditNoteFrame", "time", nil,
			{ raidid=raidid, itemid=itemid, itemitemid=itemitemid, itemtime=itemtime, itemplayer=itemplayer, what = "item" },
			{ getLootID = 1 } );
		if curRaid["Loot"][itemid]["boss"] then checked = 1 else checked = nil end
		KA_RT_Buttons:AddItem( level, "Dropped from:", curRaid["Loot"][itemid]["boss"] or "none", nil, nil, nil,
			{ opt="dropped_from_zones", cboss=curRaid["Loot"][itemid]["boss"], raidid=raidid, itemid=itemid, itemitemid=itemitemid, itemtime=itemtime, itemplayer=itemplayer },
			{ hasArrow = 1, getLootID = 1, checked = checked } );	
		
	elseif level == 2 then
		local value = this.value; 
		if not value then return end;	
		
		if value.opt == "dropped_from_zones" then
			for k, v in pairs(KA_RaidTracker_Bosses) do
				if v == 1  then
					if value.cboss == v then checked = 1 else checked = nil end
					KA_RT_Buttons:AddItem( level, k, nil, nil, nil, 
						function() KA_RT_LootItemTemplate_LootSetBoss( ) end,
						{ raidid=value.raidid, itemid=value.itemid, zone=value.zone, boss=k, itemitemid=value.itemitemid, itemtime=value.itemtime, itemplayer=value.itemplayer },
						{ checked = checked } );						
				else
					checked = nil
					if value.cboss then
						for k2, v2 in pairs(KA_RaidTracker_Bosses[k]) do
							if value["cboss"] == v2 then checked = 1; break; end
						end
					end
					KA_RT_Buttons:AddItem( level, k, nil, nil, nil, nil,
						{ opt="dropped_from_bosses", raidid=value.raidid, itemid=value.itemid, zone=k, cboss=value.cboss, itemitemid=value.itemitemid, itemtime=value.itemtime, itemplayer=value.itemplayer },
						{ hasArrow = 1, checked = checked }	);	
				end
			end
			
			if value.cboss == nil then checked = 1 else checked = nil end
			KA_RT_Buttons:AddItem( level, "None", nil, nil, nil,
				function() HideDropDownMenu(1); KA_RT_LootItemTemplate_LootSetBoss( ) end,
				value, { checked = checked } );
					
		elseif value.opt == "quick_looter" then
			for k, v in pairs(KA_RT_Buttons.QuickLooter) do
				if value.cplayer == v then checked = 1 else checked = nil end
				KA_RT_Buttons:AddItem( level, v, nil, nil, nil, 
					function() KA_RT_LootItemTemplate_LootSetLooter( ) end,				
					{ raidid=value.raidid, itemid=value.itemid, itemitemid=value.itemitemid, itemtime=value.itemtime, itemplayer=value.itemplayer, player=v, cplayer=value.cplayer },
					{ checked = checked } );						
			end

			UIDropDownMenu_AddButton( { disabled = 1 }, level );

			PlayerGroups = KA_RT_Buttons:GetPlayerGroups( value.raidid );
			for k, v in util:sortedpairs(PlayerGroups) do
				checked = nil;
				for k2, v2 in pairs(v) do
					if value.cplayer == v2 then checked = 1; break end
				end
				KA_RT_Buttons:AddItem( level, k, nil, nil, nil, nil,				
					{ opt="quick_looter_subplayers", raidid=value.raidid, itemid=value.itemid, playergroupsindex=k, itemitemid=value.itemitemid, itemtime=value.itemtime, itemplayer=value.itemplayer, players=v, cplayer=value.cplayer },
					{ hasArrow = 1, checked = checked }	);						
			end
		end

	elseif level == 3 then
		local value = this.value; 
		if not value then return end;	

		if value.opt == "dropped_from_bosses" then
			for k, v in pairs(KA_RaidTracker_Bosses[value.zone]) do
				if type(v) == "table"  then
					for k2, v2 in pairs(v) do
						if value.cboss == v then checked = 1 else checked = nil end
						KA_RT_Buttons:AddItem( level, k..' - '..v2, nil, nil, nil,
							function() KA_RT_LootItemTemplate_LootSetBoss( ) end,
							{ raidid=value.raidid, itemid=value.itemid, zone=value.zone, itemitemid=value.itemitemid, itemtime=value.itemtime, itemplayer=value.itemplayer, boss=v2 },
							{ checked = checked } );						
					end;
				else
					if value.cboss == v then checked = 1 else checked = nil end
					KA_RT_Buttons:AddItem( level, v, nil, nil, nil, 
						function() KA_RT_LootItemTemplate_LootSetBoss( ) end,
						{ raidid=value.raidid, itemid=value.itemid, zone=value.zone, itemitemid=value.itemitemid, itemtime=value.itemtime, itemplayer=value.itemplayer, boss=v },
						{ checked = checked } );						
				end;
			end
		elseif value.opt == "quick_looter_subplayers" then
			for k, v in pairs(value.players) do
				if db.RaidLog[this.value["raidid"]]["Loot"][this.value["itemid"]]["player"] == v then checked = 1 else checked = nil end
				KA_RT_Buttons:AddItem( level, v, nil, nil, nil, 
					function() KA_RT_LootItemTemplate_LootSetLooter( ) end,
					{ raidid=value.raidid, itemid=value.itemid, itemitemid=value.itemitemid, itemtime=value.itemtime, itemplayer=value.itemplayer, player=v },
					{ checked = checked } );
			end
		end

	end
end
